@@ -1,6 +1,5 @@
 # contributed by Luke Olson
 
-from __future__ import absolute_import
 import numpy as np
 
 n = 50
@@ -11,5 +10,5 @@ u = np.exp(-50 * (x**2 + y**2))
 
 from pyvisfile.vtk import write_structured_grid
 mesh = np.rollaxis(np.dstack((x, y)), 2)
-write_structured_grid('test.vts', mesh,
-        point_data=[('u', u[np.newaxis, :, :])])
+write_structured_grid("test.vts", mesh,
+        point_data=[("u", u[np.newaxis, :, :])])
